@@ -34,11 +34,12 @@ help(modelName)
     redirected (doing nothing if not redirected).
 """
 
+import importlib
 import os
 import re
 import sys
 from contextlib import suppress
-
+from types import ModuleType
 
 def _get_amici_path():
     """
@@ -119,6 +120,12 @@ except (ImportError, AttributeError) as e:
         # Probably some linking problem that we don't want to hide
         raise e
 
+from typing import Optional, Union, Sequence, List
+AmiciModel = Union['amici.Model', 'amici.ModelPtr']
+AmiciSolver = Union['amici.Solver', 'amici.SolverPtr']
+AmiciExpData = Union['amici.ExpData', 'amici.ExpDataPtr']
+AmiciExpDataVector = Union['amici.ExpDataPtrVector', Sequence[AmiciExpData]]
+
 # Initialize AMICI paths
 amici_path = _get_amici_path()
 amiciSwigPath = os.path.join(amici_path, 'swig')
@@ -150,21 +157,21 @@ except (ModuleNotFoundError, ImportError) as e:
         print(f'Suppressing error {str(e)}')
 
 
-def runAmiciSimulation(model, solver, edata=None):
-    """ Convenience wrapper around amici.runAmiciSimulation (generated by swig)
-
-    Arguments:
-        model: Model instance
-        solver: Solver instance, must be generated from Model.getSolver()
-        edata: ExpData instance (optional)
-
-    Returns:
-        ReturnData object with simulation results
-
-    Raises:
-
+def runAmiciSimulation(
+        model: AmiciModel,
+        solver: AmiciSolver,
+        edata: Optional[AmiciExpData] = None
+) -> 'numpy.ReturnDataView':
     """
-    if edata and edata.__class__.__name__ == 'ExpDataPtr':
+    Convenience wrapper around amici.runAmiciSimulation (generated by swig)
+
+    :param model: Model instance
+`   :param solver: Solver instance, must be generated from Model.getSolver()
+    :param edata: ExpData instance (optional)
+
+    :returns: ReturnData object with simulation results
+    """
+    if edata and isinstance(edata, amici.ExpDataPtr):
         edata = edata.get()
 
     with capture_cstdout():
@@ -172,48 +179,45 @@ def runAmiciSimulation(model, solver, edata=None):
     return numpy.ReturnDataView(rdata)
 
 
-def ExpData(*args):
-    """ Convenience wrapper for ExpData constructors
+def ExpData(*args) -> 'amici.ExpData':
+    """
+    Convenience wrapper for ExpData constructors
 
-    Arguments:
-        args: arguments
-
-    Returns:
-        ExpData Instance
-
-    Raises:
+    :param args: arguments
 
+    :returns: ExpData Instance
     """
     if isinstance(args[0], ReturnDataView):
         return amici.ExpData(args[0]['ptr'].get(), *args[1:])
-    elif isinstance(args[0], ExpDataPtr):
+    elif isinstance(args[0], amici.ExpDataPtr):
         # the *args[:1] should be empty, but by the time you read this,
         # the constructor signature may have changed and you are glad this
         # wrapper did not break.
-        return amici.ExpData(args[0].get(), *args[:1])
-    elif isinstance(args[0], ModelPtr):
+        return amici.ExpData(args[0].get(), *args[1:])
+    elif isinstance(args[0], amici.ModelPtr):
         return amici.ExpData(args[0].get())
     else:
         return amici.ExpData(*args)
 
 
-def runAmiciSimulations(model, solver, edata_list, failfast=True,
-                        num_threads=1):
-    """ Convenience wrapper for loops of amici.runAmiciSimulation
+def runAmiciSimulations(
+        model: AmiciModel,
+        solver: AmiciSolver,
+        edata_list: AmiciExpDataVector,
+        failfast: bool = True,
+        num_threads: int = 1,
+) -> List['numpy.ReturnDataView']:
+    """
+    Convenience wrapper for loops of amici.runAmiciSimulation
 
-    Arguments:
-        model: Model instance
-        solver: Solver instance, must be generated from Model.getSolver()
-        edata_list: list of ExpData instances
-        failfast: returns as soon as an integration failure is encountered
-        num_threads: number of threads to use
-                     (only used if compiled with openmp)
-
-    Returns:
-        list of ReturnData objects with simulation results
-
-    Raises:
+    :param model: Model instance
+    :param solver: Solver instance, must be generated from Model.getSolver()
+    :param edata_list: list of ExpData instances
+    :param failfast: returns as soon as an integration failure is encountered
+    :param num_threads: number of threads to use (only used if compiled
+        with openmp)
 
+    :returns: list of simulation results
     """
     with capture_cstdout():
         edata_ptr_vector = amici.ExpDataPtrVector(edata_list)
@@ -223,3 +227,80 @@ def runAmiciSimulations(model, solver, edata_list, failfast=True,
                                                    failfast,
                                                    num_threads)
     return [numpy.ReturnDataView(r) for r in rdata_ptr_list]
+
+
+def readSolverSettingsFromHDF5(
+        file: str,
+        solver: AmiciSolver,
+        location: Optional[str] = 'solverSettings'
+) -> None:
+    """
+    Convenience wrapper for :fun:`amici.readSolverSettingsFromHDF5`
+
+    :param file: hdf5 filename
+    :param solver: Solver instance to which settings will be transferred
+    :param location: location of solver settings in hdf5 file
+    """
+    if isinstance(solver, amici.SolverPtr):
+        amici.readSolverSettingsFromHDF5(file, solver.get(), location)
+    else:
+        amici.readSolverSettingsFromHDF5(file, solver, location)
+
+
+def writeSolverSettingsToHDF5(
+        solver: AmiciSolver,
+        file: Union[str, object],
+        location: Optional[str] = 'solverSettings'
+) -> None:
+    """
+    Convenience wrapper for :fun:`amici.writeSolverSettingsToHDF5`
+
+    :param file: hdf5 filename, can also be object created by
+        :fun:`amici.createOrOpenForWriting`
+    :param solver: Solver instance from which settings will stored
+    :param location: location of solver settings in hdf5 file
+    """
+    if isinstance(solver, amici.SolverPtr):
+        amici.writeSolverSettingsToHDF5(solver.get(), file, location)
+    else:
+        amici.writeSolverSettingsToHDF5(solver, file, location)
+
+
+class add_path:
+    """Context manager for temporarily changing PYTHONPATH"""
+    def __init__(self, path: str):
+        self.path: str = path
+
+    def __enter__(self):
+        if self.path:
+            sys.path.insert(0, self.path)
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        try:
+            sys.path.remove(self.path)
+        except ValueError:
+            pass
+
+
+def import_model_module(module_name: str,
+                        module_path: Optional[str] = None) -> ModuleType:
+    """
+    Import Python module of an AMICI model
+
+    :param module_name: Name of the python module for the model
+    :param module_path: Absolute path of the module
+    :return: The model module
+    """
+
+    # ensure we will find the newly created module
+    importlib.invalidate_caches()
+
+    with add_path(module_path):
+        # module already loaded?
+        if module_name in sys.modules:
+            # reload, because may just have been created
+            importlib.reload(sys.modules[module_name])
+            return sys.modules[module_name]
+
+
+        return importlib.import_module(module_name)
