@@ -6,8 +6,8 @@
 #include "amici/vector.h"
 
 #include <sunmatrix/sunmatrix_band.h>
-#include <sunmatrix/sunmatrix_sparse.h>
 #include <sunmatrix/sunmatrix_dense.h>
+#include <sunmatrix/sunmatrix_sparse.h>
 
 #include <memory>
 
@@ -24,38 +24,37 @@ class Solver;
  */
 class AbstractModel {
   public:
-
     virtual ~AbstractModel() = default;
 
     /**
-     * Retrieves the solver object
+     * @brief Retrieves the solver object
      * @return The Solver instance
      */
     virtual std::unique_ptr<Solver> getSolver() = 0;
 
     /**
-     * Root function
+     * @brief Root function
      * @param t time
      * @param x state
      * @param dx time derivative of state (DAE only)
      * @param root array to which values of the root function will be written
      */
-    virtual void froot(realtype t, AmiVector *x, AmiVector *dx,
-                       realtype *root) = 0;
+    virtual void froot(const realtype t, const AmiVector &x,
+                       const AmiVector &dx, gsl::span<realtype> root) = 0;
 
     /**
-     * Residual function
+     * @brief Residual function
      * @param t time
      * @param x state
      * @param dx time derivative of state (DAE only)
      * @param xdot array to which values of the residual function will be
      * written
      */
-    virtual void fxdot(realtype t, AmiVector *x, AmiVector *dx,
-                       AmiVector *xdot) = 0;
+    virtual void fxdot(const realtype t, const AmiVector &x,
+                       const AmiVector &dx, AmiVector &xdot) = 0;
 
     /**
-     * Sensitivity Residual function
+     * @brief Sensitivity Residual function
      * @param t time
      * @param x state
      * @param dx time derivative of state (DAE only)
@@ -65,11 +64,12 @@ class AbstractModel {
      * @param sxdot array to which values of the sensitivity residual function
      * will be written
      */
-    virtual void fsxdot(realtype t, AmiVector *x, AmiVector *dx, int ip,
-                        AmiVector *sx, AmiVector *sdx, AmiVector *sxdot) = 0;
+    virtual void fsxdot(const realtype t, const AmiVector &x,
+                        const AmiVector &dx, int ip, const AmiVector &sx,
+                        const AmiVector &sdx, AmiVector &sxdot) = 0;
 
     /**
-     * Dense Jacobian function
+     * @brief Dense Jacobian function
      * @param t time
      * @param cj scaling factor (inverse of timestep, DAE only)
      * @param x state
@@ -77,11 +77,12 @@ class AbstractModel {
      * @param xdot values of residual function (unused)
      * @param J dense matrix to which values of the jacobian will be written
      */
-    virtual void fJ(realtype t, realtype cj, AmiVector *x, AmiVector *dx,
-                    AmiVector *xdot, SUNMatrix J) = 0;
+    virtual void fJ(const realtype t, realtype cj, const AmiVector &x,
+                    const AmiVector &dx, const AmiVector &xdot,
+                    SUNMatrix J) = 0;
 
     /**
-     * Sparse Jacobian function
+     * @brief Sparse Jacobian function
      * @param t time
      * @param cj scaling factor (inverse of timestep, DAE only)
      * @param x state
@@ -89,32 +90,33 @@ class AbstractModel {
      * @param xdot values of residual function (unused)
      * @param J sparse matrix to which values of the Jacobian will be written
      */
-    virtual void fJSparse(realtype t, realtype cj, AmiVector *x, AmiVector *dx,
-                          AmiVector *xdot, SUNMatrix J) = 0;
+    virtual void fJSparse(const realtype t, realtype cj, const AmiVector &x,
+                          const AmiVector &dx, const AmiVector &xdot,
+                          SUNMatrix J) = 0;
 
     /**
-     * Diagonal Jacobian function
+     * @brief Diagonal Jacobian function
      * @param t time
      * @param Jdiag array to which the diagonal of the Jacobian will be written
      * @param cj scaling factor (inverse of timestep, DAE only)
      * @param x state
      * @param dx time derivative of state (DAE only)
-     * @return flag indicating successful evaluation
      */
-    virtual void fJDiag(realtype t, AmiVector *Jdiag, realtype cj, AmiVector *x,
-                        AmiVector *dx) = 0;
+    virtual void fJDiag(const realtype t, AmiVector &Jdiag, realtype cj,
+                        const AmiVector &x, const AmiVector &dx) = 0;
 
     /**
-     * Parameter derivative of residual function
+     * @brief Model specific sparse implementation of explicit parameter
+     * derivative of right hand side
      * @param t time
      * @param x state
      * @param dx time derivative of state (DAE only)
-     * @return flag indicating successful evaluation
      */
-    virtual void fdxdotdp(realtype t, AmiVector *x, AmiVector *dx) = 0;
+    virtual void fdxdotdp(const realtype t, const AmiVector &x,
+                          const AmiVector &dx) = 0;
 
     /**
-     * Jacobian multiply function
+     * @brief Jacobian multiply function
      * @param t time
      * @param x state
      * @param dx time derivative of state (DAE only)
@@ -123,8 +125,9 @@ class AbstractModel {
      * @param nJv array to which result of multiplication will be written
      * @param cj scaling factor (inverse of timestep, DAE only)
      */
-    virtual void fJv(realtype t, AmiVector *x, AmiVector *dx, AmiVector *xdot,
-                     AmiVector *v, AmiVector *nJv, realtype cj) = 0;
+    virtual void fJv(const realtype t, const AmiVector &x, const AmiVector &dx,
+                     const AmiVector &xdot, const AmiVector &v, AmiVector &nJv,
+                     realtype cj) = 0;
 
     /**
      * @brief Returns the amici version that was used to generate the model
@@ -144,13 +147,13 @@ class AbstractModel {
      * @param t initial time
      * @param p parameter vector
      * @param k constant vector
-     **/
+     */
     virtual void fx0(realtype *x0, const realtype t, const realtype *p,
                      const realtype *k);
 
     /**
-     * Function indicating whether reinitialization of states depending on
-     * fixed parameters is permissible
+     * @brief Function indicating whether reinitialization of states depending
+     * on fixed parameters is permissible
      * @return flag inidication whether reinitialization of states depending on
      * fixed parameters is permissible
      */
@@ -162,7 +165,7 @@ class AbstractModel {
      * @param t initial time
      * @param p parameter vector
      * @param k constant vector
-     **/
+     */
     virtual void fx0_fixedParameters(realtype *x0, const realtype t,
                                      const realtype *p, const realtype *k);
 
@@ -174,10 +177,10 @@ class AbstractModel {
      * @param p parameter vector
      * @param k constant vector
      * @param ip sensitivity index
-     **/
+     */
     virtual void fsx0_fixedParameters(realtype *sx0, const realtype t,
                                       const realtype *x0, const realtype *p,
-                                      const realtype *k, const int ip);
+                                      const realtype *k, int ip);
 
     /**
      * @brief Model specific implementation of fsx0
@@ -187,17 +190,18 @@ class AbstractModel {
      * @param p parameter vector
      * @param k constant vector
      * @param ip sensitivity index
-     **/
+     */
     virtual void fsx0(realtype *sx0, const realtype t, const realtype *x0,
-                      const realtype *p, const realtype *k, const int ip);
+                      const realtype *p, const realtype *k, int ip);
 
     /**
-     * Initial value for time derivative of states (only necessary for DAEs)
+     * @brief Initial value for time derivative of states (only necessary for
+     * DAEs)
      * @param x0 Vector with the initial states
-     * @param dx0 Vector to which the initial derivative states will be
-     * written (only DAE)
-     **/
-    virtual void fdx0(AmiVector *x0, AmiVector *dx0);
+     * @param dx0 Vector to which the initial derivative states will be written
+     * (only DAE)
+     */
+    virtual void fdx0(AmiVector &x0, AmiVector &dx0);
 
     /**
      * @brief Model specific implementation of fstau
@@ -210,10 +214,10 @@ class AbstractModel {
      * @param sx current state sensitivity
      * @param ip sensitivity index
      * @param ie event index
-     **/
+     */
     virtual void fstau(realtype *stau, const realtype t, const realtype *x,
                        const realtype *p, const realtype *k, const realtype *h,
-                       const realtype *sx, const int ip, const int ie);
+                       const realtype *sx, int ip, int ie);
 
     /**
      * @brief Model specific implementation of fy
@@ -224,7 +228,7 @@ class AbstractModel {
      * @param k constant vector
      * @param h heavyside vector
      * @param w repeating elements vector
-     **/
+     */
     virtual void fy(realtype *y, const realtype t, const realtype *x,
                     const realtype *p, const realtype *k, const realtype *h,
                     const realtype *w);
@@ -240,10 +244,10 @@ class AbstractModel {
      * @param ip parameter index w.r.t. which the derivative is requested
      * @param w repeating elements vector
      * @param dwdp Recurring terms in xdot, parameter derivative
-     **/
+     */
     virtual void fdydp(realtype *dydp, const realtype t, const realtype *x,
                        const realtype *p, const realtype *k, const realtype *h,
-                       const int ip, const realtype *w, const realtype *dwdp);
+                       int ip, const realtype *w, const realtype *dwdp);
 
     /**
      * @brief Model specific implementation of fdydx
@@ -255,7 +259,7 @@ class AbstractModel {
      * @param h heavyside vector
      * @param w repeating elements vector
      * @param dwdx Recurring terms in xdot, state derivative
-     **/
+     */
     virtual void fdydx(realtype *dydx, const realtype t, const realtype *x,
                        const realtype *p, const realtype *k, const realtype *h,
                        const realtype *w, const realtype *dwdx);
@@ -269,10 +273,9 @@ class AbstractModel {
      * @param p parameter vector
      * @param k constant vector
      * @param h heavyside vector
-     **/
-    virtual void fz(realtype *z, const int ie, const realtype t,
-                    const realtype *x, const realtype *p, const realtype *k,
-                    const realtype *h);
+     */
+    virtual void fz(realtype *z, int ie, const realtype t, const realtype *x,
+                    const realtype *p, const realtype *k, const realtype *h);
 
     /**
      * @brief Model specific implementation of fsz
@@ -285,25 +288,24 @@ class AbstractModel {
      * @param h heavyside vector
      * @param sx current state sensitivity
      * @param ip sensitivity index
-     **/
-    virtual void fsz(realtype *sz, const int ie, const realtype t,
-                     const realtype *x, const realtype *p, const realtype *k,
-                     const realtype *h, const realtype *sx, const int ip);
+     */
+    virtual void fsz(realtype *sz, int ie, const realtype t, const realtype *x,
+                     const realtype *p, const realtype *k, const realtype *h,
+                     const realtype *sx, int ip);
 
     /**
      * @brief Model specific implementation of frz
      * @param rz value of root function at current timepoint (non-output events
-     *not included)
+     * not included)
      * @param ie event index
      * @param t current time
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
      * @param h heavyside vector
-     **/
-    virtual void frz(realtype *rz, const int ie, const realtype t,
-                     const realtype *x, const realtype *p, const realtype *k,
-                     const realtype *h);
+     */
+    virtual void frz(realtype *rz, int ie, const realtype t, const realtype *x,
+                     const realtype *p, const realtype *k, const realtype *h);
 
     /**
      * @brief Model specific implementation of fsrz
@@ -316,15 +318,15 @@ class AbstractModel {
      * @param sx current state sensitivity
      * @param h heavyside vector
      * @param ip sensitivity index
-     **/
-    virtual void fsrz(realtype *srz, const int ie, const realtype t,
+     */
+    virtual void fsrz(realtype *srz, int ie, const realtype t,
                       const realtype *x, const realtype *p, const realtype *k,
-                      const realtype *h, const realtype *sx, const int ip);
+                      const realtype *h, const realtype *sx, int ip);
 
     /**
      * @brief Model specific implementation of fdzdp
      * @param dzdp partial derivative of event-resolved output z w.r.t. model
-     *parameters p
+     * parameters p
      * @param ie event index
      * @param t current time
      * @param x current state
@@ -332,30 +334,30 @@ class AbstractModel {
      * @param k constant vector
      * @param h heavyside vector
      * @param ip parameter index w.r.t. which the derivative is requested
-     **/
-    virtual void fdzdp(realtype *dzdp, const int ie, const realtype t,
+     */
+    virtual void fdzdp(realtype *dzdp, int ie, const realtype t,
                        const realtype *x, const realtype *p, const realtype *k,
-                       const realtype *h, const int ip);
+                       const realtype *h, int ip);
 
     /**
      * @brief Model specific implementation of fdzdx
      * @param dzdx partial derivative of event-resolved output z w.r.t. model
-     *states x
+     * states x
      * @param ie event index
      * @param t current time
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
      * @param h heavyside vector
-     **/
-    virtual void fdzdx(realtype *dzdx, const int ie, const realtype t,
+     */
+    virtual void fdzdx(realtype *dzdx, int ie, const realtype t,
                        const realtype *x, const realtype *p, const realtype *k,
                        const realtype *h);
 
     /**
      * @brief Model specific implementation of fdrzdp
      * @param drzdp partial derivative of root output rz w.r.t. model parameters
-     *p
+     * p
      * @param ie event index
      * @param t current time
      * @param x current state
@@ -363,10 +365,10 @@ class AbstractModel {
      * @param k constant vector
      * @param h heavyside vector
      * @param ip parameter index w.r.t. which the derivative is requested
-     **/
-    virtual void fdrzdp(realtype *drzdp, const int ie, const realtype t,
+     */
+    virtual void fdrzdp(realtype *drzdp, int ie, const realtype t,
                         const realtype *x, const realtype *p, const realtype *k,
-                        const realtype *h, const int ip);
+                        const realtype *h, int ip);
 
     /**
      * @brief Model specific implementation of fdrzdx
@@ -377,8 +379,8 @@ class AbstractModel {
      * @param p parameter vector
      * @param k constant vector
      * @param h heavyside vector
-     **/
-    virtual void fdrzdx(realtype *drzdx, const int ie, const realtype t,
+     */
+    virtual void fdrzdx(realtype *drzdx, int ie, const realtype t,
                         const realtype *x, const realtype *p, const realtype *k,
                         const realtype *h);
 
@@ -393,10 +395,10 @@ class AbstractModel {
      * @param ie event index
      * @param xdot new model right hand side
      * @param xdot_old previous model right hand side
-     **/
+     */
     virtual void fdeltax(realtype *deltax, const realtype t, const realtype *x,
                          const realtype *p, const realtype *k,
-                         const realtype *h, const int ie, const realtype *xdot,
+                         const realtype *h, int ie, const realtype *xdot,
                          const realtype *xdot_old);
 
     /**
@@ -414,11 +416,11 @@ class AbstractModel {
      * @param xdot_old previous model right hand side
      * @param sx state sensitivity
      * @param stau event-time sensitivity
-     **/
+     */
     virtual void fdeltasx(realtype *deltasx, const realtype t,
                           const realtype *x, const realtype *p,
                           const realtype *k, const realtype *h,
-                          const realtype *w, const int ip, const int ie,
+                          const realtype *w, int ip, int ie,
                           const realtype *xdot, const realtype *xdot_old,
                           const realtype *sx, const realtype *stau);
 
@@ -434,10 +436,10 @@ class AbstractModel {
      * @param xdot new model right hand side
      * @param xdot_old previous model right hand side
      * @param xB current adjoint state
-     **/
+     */
     virtual void fdeltaxB(realtype *deltaxB, const realtype t,
                           const realtype *x, const realtype *p,
-                          const realtype *k, const realtype *h, const int ie,
+                          const realtype *k, const realtype *h, int ie,
                           const realtype *xdot, const realtype *xdot_old,
                           const realtype *xB);
 
@@ -454,12 +456,12 @@ class AbstractModel {
      * @param xdot new model right hand side
      * @param xdot_old previous model right hand side
      * @param xB adjoint state
-     **/
+     */
     virtual void fdeltaqB(realtype *deltaqB, const realtype t,
                           const realtype *x, const realtype *p,
-                          const realtype *k, const realtype *h, const int ip,
-                          const int ie, const realtype *xdot,
-                          const realtype *xdot_old, const realtype *xB);
+                          const realtype *k, const realtype *h, int ip, int ie,
+                          const realtype *xdot, const realtype *xdot_old,
+                          const realtype *xB);
 
     /**
      * @brief Model specific implementation of fsigmay
@@ -467,7 +469,7 @@ class AbstractModel {
      * @param t current time
      * @param p parameter vector
      * @param k constant vector
-     **/
+     */
     virtual void fsigmay(realtype *sigmay, const realtype t, const realtype *p,
                          const realtype *k);
 
@@ -478,9 +480,9 @@ class AbstractModel {
      * @param p parameter vector
      * @param k constant vector
      * @param ip sensitivity index
-     **/
+     */
     virtual void fdsigmaydp(realtype *dsigmaydp, const realtype t,
-                            const realtype *p, const realtype *k, const int ip);
+                            const realtype *p, const realtype *k, int ip);
 
     /**
      * @brief Model specific implementation of fsigmaz
@@ -488,21 +490,21 @@ class AbstractModel {
      * @param t current time
      * @param p parameter vector
      * @param k constant vector
-     **/
+     */
     virtual void fsigmaz(realtype *sigmaz, const realtype t, const realtype *p,
                          const realtype *k);
 
     /**
      * @brief Model specific implementation of fsigmaz
      * @param dsigmazdp partial derivative of standard deviation of event
-     *measurements
+     * measurements
      * @param t current time
      * @param p parameter vector
      * @param k constant vector
      * @param ip sensitivity index
-     **/
+     */
     virtual void fdsigmazdp(realtype *dsigmazdp, const realtype t,
-                            const realtype *p, const realtype *k, const int ip);
+                            const realtype *p, const realtype *k, int ip);
 
     /**
      * @brief Model specific implementation of fJy
@@ -513,8 +515,8 @@ class AbstractModel {
      * @param y model output at timepoint
      * @param sigmay measurement standard deviation at timepoint
      * @param my measurements at timepoint
-     **/
-    virtual void fJy(realtype *nllh, const int iy, const realtype *p,
+     */
+    virtual void fJy(realtype *nllh, int iy, const realtype *p,
                      const realtype *k, const realtype *y,
                      const realtype *sigmay, const realtype *my);
 
@@ -527,8 +529,8 @@ class AbstractModel {
      * @param z model event output at timepoint
      * @param sigmaz event measurement standard deviation at timepoint
      * @param mz event measurements at timepoint
-     **/
-    virtual void fJz(realtype *nllh, const int iz, const realtype *p,
+     */
+    virtual void fJz(realtype *nllh, int iz, const realtype *p,
                      const realtype *k, const realtype *z,
                      const realtype *sigmaz, const realtype *mz);
 
@@ -540,72 +542,70 @@ class AbstractModel {
      * @param k constant vector
      * @param z model event output at timepoint
      * @param sigmaz event measurement standard deviation at timepoint
-     **/
-    virtual void fJrz(realtype *nllh, const int iz, const realtype *p,
+     */
+    virtual void fJrz(realtype *nllh, int iz, const realtype *p,
                       const realtype *k, const realtype *z,
                       const realtype *sigmaz);
 
     /**
      * @brief Model specific implementation of fdJydy
      * @param dJydy partial derivative of time-resolved measurement negative
-     *log-likelihood Jy
+     * log-likelihood Jy
      * @param iy output index
      * @param p parameter vector
      * @param k constant vector
      * @param y model output at timepoint
      * @param sigmay measurement standard deviation at timepoint
      * @param my measurement at timepoint
-     **/
-    virtual void fdJydy(realtype *dJydy, const int iy, const realtype *p,
+     */
+    virtual void fdJydy(realtype *dJydy, int iy, const realtype *p,
                         const realtype *k, const realtype *y,
                         const realtype *sigmay, const realtype *my);
 
     /**
      * @brief Model specific implementation of fdJydsigma
-     * @param dJydsigma Sensitivity of time-resolved measurement
-     * negative log-likelihood Jy w.r.t. standard deviation sigmay
+     * @param dJydsigma Sensitivity of time-resolved measurement negative
+     * log-likelihood Jy w.r.t. standard deviation sigmay
      * @param iy output index
      * @param p parameter vector
      * @param k constant vector
      * @param y model output at timepoint
      * @param sigmay measurement standard deviation at timepoint
      * @param my measurement at timepoint
-     **/
-    virtual void fdJydsigma(realtype *dJydsigma, const int iy,
-                            const realtype *p, const realtype *k,
-                            const realtype *y, const realtype *sigmay,
-                            const realtype *my);
+     */
+    virtual void fdJydsigma(realtype *dJydsigma, int iy, const realtype *p,
+                            const realtype *k, const realtype *y,
+                            const realtype *sigmay, const realtype *my);
 
     /**
-     *@brief Model specific implementation of fdJzdz
+     * @brief Model specific implementation of fdJzdz
      * @param dJzdz partial derivative of event measurement negative
-     *log-likelihood Jz
+     * log-likelihood Jz
      * @param iz event output index
      * @param p parameter vector
      * @param k constant vector
      * @param z model event output at timepoint
      * @param sigmaz event measurement standard deviation at timepoint
      * @param mz event measurement at timepoint
-     **/
-    virtual void fdJzdz(realtype *dJzdz, const int iz, const realtype *p,
+     */
+    virtual void fdJzdz(realtype *dJzdz, int iz, const realtype *p,
                         const realtype *k, const realtype *z,
                         const realtype *sigmaz, const realtype *mz);
 
     /**
      * @brief Model specific implementation of fdJzdsigma
-     * @param dJzdsigma Sensitivity of event measurement
-     * negative log-likelihood Jz w.r.t. standard deviation sigmaz
+     * @param dJzdsigma Sensitivity of event measurement negative log-likelihood
+     * Jz w.r.t. standard deviation sigmaz
      * @param iz event output index
      * @param p parameter vector
      * @param k constant vector
      * @param z model event output at timepoint
      * @param sigmaz event measurement standard deviation at timepoint
      * @param mz event measurement at timepoint
-     **/
-    virtual void fdJzdsigma(realtype *dJzdsigma, const int iz,
-                            const realtype *p, const realtype *k,
-                            const realtype *z, const realtype *sigmaz,
-                            const realtype *mz);
+     */
+    virtual void fdJzdsigma(realtype *dJzdsigma, int iz, const realtype *p,
+                            const realtype *k, const realtype *z,
+                            const realtype *sigmaz, const realtype *mz);
 
     /**
      * @brief Model specific implementation of fdJrzdz
@@ -615,24 +615,24 @@ class AbstractModel {
      * @param k constant vector
      * @param rz model root output at timepoint
      * @param sigmaz event measurement standard deviation at timepoint
-     **/
-    virtual void fdJrzdz(realtype *dJrzdz, const int iz, const realtype *p,
+     */
+    virtual void fdJrzdz(realtype *dJrzdz, int iz, const realtype *p,
                          const realtype *k, const realtype *rz,
                          const realtype *sigmaz);
 
     /**
      * @brief Model specific implementation of fdJrzdsigma
-     * @param dJrzdsigma Sensitivity of event penalization Jrz w.r.t.
-     * standard deviation sigmaz
+     * @param dJrzdsigma Sensitivity of event penalization Jrz w.r.t. standard
+     * deviation sigmaz
      * @param iz event output index
      * @param p parameter vector
      * @param k constant vector
      * @param rz model root output at timepoint
      * @param sigmaz event measurement standard deviation at timepoint
-     **/
-    virtual void fdJrzdsigma(realtype *dJrzdsigma, const int iz,
-                             const realtype *p, const realtype *k,
-                             const realtype *rz, const realtype *sigmaz);
+     */
+    virtual void fdJrzdsigma(realtype *dJrzdsigma, int iz, const realtype *p,
+                             const realtype *k, const realtype *rz,
+                             const realtype *sigmaz);
 
     /**
      * @brief Model specific implementation of fw
@@ -665,6 +665,18 @@ class AbstractModel {
                        const realtype *w, const realtype *tcl,
                        const realtype *stcl);
 
+    /**
+     * @brief Model specific implementation for dwdp, column pointers
+     * @param indexptrs column pointers
+     */
+    virtual void fdwdp_colptrs(sunindextype *indexptrs);
+
+    /**
+     * @brief Model specific implementation for dwdp, row values
+     * @param indexvals row values
+     */
+    virtual void fdwdp_rowvals(sunindextype *indexvals);
+
     /**
      * @brief Model specific sensitivity implementation of dwdp
      * @param dwdp Recurring terms in xdot, parameter derivative
@@ -681,7 +693,7 @@ class AbstractModel {
     virtual void fdwdp(realtype *dwdp, const realtype t, const realtype *x,
                        const realtype *p, const realtype *k, const realtype *h,
                        const realtype *w, const realtype *tcl,
-                       const realtype *stcl, const int ip);
+                       const realtype *stcl, int ip);
 
     /**
      * @brief Model specific implementation of dwdx, data part
@@ -701,13 +713,13 @@ class AbstractModel {
     /**
      * @brief Model specific implementation for dwdx, column pointers
      * @param indexptrs column pointers
-     **/
+     */
     virtual void fdwdx_colptrs(sunindextype *indexptrs);
 
     /**
      * @brief Model specific implementation for dwdx, row values
      * @param indexvals row values
-     **/
+     */
     virtual void fdwdx_rowvals(sunindextype *indexvals);
 };
 
