@@ -3,6 +3,7 @@
 
 #include "amici/defines.h"
 #include "amici/vector.h"
+#include "amici/misc.h"
 
 #include <vector>
 
@@ -520,7 +521,7 @@ void checkSigmaPositivity(realtype sigma, const char *sigmaName);
  * @brief The ConditionContext class applies condition-specific amici::Model
  * settings and restores them when going out of scope
  */
-class ConditionContext {
+class ConditionContext : public ContextManager {
   public:
     /**
      * @brief Apply condition-specific settings from edata to model while
@@ -528,8 +529,13 @@ class ConditionContext {
      *
      * @param model
      * @param edata
+     * @param fpc flag indicating which fixedParmeter from edata to apply
      */
-    explicit ConditionContext(Model *model, const ExpData *edata = nullptr);
+    explicit ConditionContext(
+        Model *model, const ExpData *edata = nullptr,
+        FixedParameterContext fpc = FixedParameterContext::simulation);
+
+    ConditionContext &operator=(const ConditionContext &other) = delete;
 
     ~ConditionContext();
 
@@ -539,8 +545,10 @@ class ConditionContext {
      * backed-up in the constructor call.
      *
      * @param edata
+     * @param fpc flag indicating which fixedParmeter from edata to apply
      */
-    void applyCondition(const ExpData *edata);
+    void applyCondition(const ExpData *edata,
+                        FixedParameterContext fpc);
 
     /**
      * @brief Restore original settings on constructor-supplied amici::Model.
