@@ -8,12 +8,10 @@
 
 #include <memory>
 
-
-
 namespace amici {
 
-class ReturnData;
 class Model;
+class Solver;
 class AmiVector;
 
 /**
@@ -25,43 +23,28 @@ class NewtonSolver {
 
   public:
     /**
-     * Initializes all members with the provided objects
+     * @brief Initializes all members with the provided objects
      *
      * @param t pointer to time variable
      * @param x pointer to state variables
-     * @param model pointer to the AMICI model object
-     * @param rdata pointer to the return data object
+     * @param model pointer to the model object
      */
-    NewtonSolver(realtype *t, AmiVector *x, Model *model, ReturnData *rdata);
+    NewtonSolver(realtype *t, AmiVector *x, Model *model);
 
     /**
-     * Factory method to create a NewtonSolver based on linsolType
+     * @brief Factory method to create a NewtonSolver based on linsolType
      *
      * @param t pointer to time variable
      * @param x pointer to state variables
-     * @param linsolType integer indicating which linear solver to use
-     * @param model pointer to the AMICI model object
-     * @param rdata pointer to the return data object
-     * @param maxlinsteps maximum number of allowed linear steps per Newton step for steady state computation
-     * @param maxsteps maximum number of allowed Newton steps for steady state computation
-     * @param atol absolute tolerance
-     * @param rtol relative tolerance
-     * @param dampingFactorMode to switch on/off a use of a damping factor
-     * @param dampingFactorLowerBound a lower bound for the damping factor
+     * @param simulationSolver solver with settings
+     * @param model pointer to the model object
      * @return solver NewtonSolver according to the specified linsolType
      */
-    static std::unique_ptr<NewtonSolver> getSolver(realtype *t, AmiVector *x,
-                                                   LinearSolver linsolType,
-                                                   Model *model,
-                                                   ReturnData *rdata,
-                                                   int maxlinsteps,
-                                                   int maxsteps,
-                                                   double atol, double rtol,
-                                                   NewtonDampingFactorMode dampingFactorMode,
-                                                   double dampingFactorLowerBound);
+    static std::unique_ptr<NewtonSolver> getSolver(
+    realtype *t, AmiVector *x, Solver &simulationSolver, Model *model);
 
     /**
-     * Computes the solution of one Newton iteration
+     * @brief Computes the solution of one Newton iteration
      *
      * @param ntry integer newton_try integer start number of Newton solver
      * (1 or 2)
@@ -72,14 +55,23 @@ class NewtonSolver {
     void getStep(int ntry, int nnewt, AmiVector &delta);
 
     /**
-     * Computes steady state sensitivities
+     * @brief Computes steady state sensitivities
      *
      * @param sx pointer to state variable sensitivities
      */
     void computeNewtonSensis(AmiVectorArray &sx);
+    
+    /**
+     * @brief Accessor for numlinsteps
+     *
+     * @return numlinsteps
+     */
+    const std::vector<int> &getNumLinSteps() const {
+        return numlinsteps;
+    }
 
     /**
-     * Writes the Jacobian for the Newton iteration and passes it to the linear
+     * @brief Writes the Jacobian for the Newton iteration and passes it to the linear
      * solver
      *
      * @param ntry integer newton_try integer start number of Newton solver
@@ -89,7 +81,7 @@ class NewtonSolver {
     virtual void prepareLinearSystem(int ntry, int nnewt) = 0;
 
     /**
-     * Solves the linear system for the Newton step
+     * @brief Solves the linear system for the Newton step
      *
      * @param rhs containing the RHS of the linear system, will be
      * overwritten by solution to the linear system
@@ -115,17 +107,17 @@ class NewtonSolver {
   protected:
     /** time variable */
     realtype *t;
-    /** pointer to the AMICI model object */
+    /** pointer to the model object */
     Model *model;
-    /** pointer to the return data object */
-    ReturnData *rdata;
     /** right hand side AmiVector */
     AmiVector xdot;
     /** current state */
     AmiVector *x;
     /** current state time derivative (DAE) */
     AmiVector dx;
-
+    
+    /** history of number of linear steps */
+    std::vector<int> numlinsteps;
 };
 
 /**
@@ -137,20 +129,19 @@ class NewtonSolverDense : public NewtonSolver {
 
   public:
     /**
-     * Constructor, initializes all members with the provided objects
+     * @brief Constructor, initializes all members with the provided objects
      * and initializes temporary storage objects
      *
      * @param t pointer to time variable
      * @param x pointer to state variables
-     * @param model pointer to the AMICI model object
-     * @param rdata pointer to the return data object
+     * @param model pointer to the model object
      */
 
-    NewtonSolverDense(realtype *t, AmiVector *x, Model *model, ReturnData *rdata);
+    NewtonSolverDense(realtype *t, AmiVector *x, Model *model);
     ~NewtonSolverDense() override;
 
     /**
-     * Solves the linear system for the Newton step
+     * @brief Solves the linear system for the Newton step
      *
      * @param rhs containing the RHS of the linear system, will be
      * overwritten by solution to the linear system
@@ -158,7 +149,7 @@ class NewtonSolverDense : public NewtonSolver {
     void solveLinearSystem(AmiVector &rhs) override;
 
     /**
-     * Writes the Jacobian for the Newton iteration and passes it to the linear
+     * @brief Writes the Jacobian for the Newton iteration and passes it to the linear
      * solver
      *
      * @param ntry integer newton_try integer start number of Newton solver
@@ -184,19 +175,18 @@ class NewtonSolverSparse : public NewtonSolver {
 
   public:
     /**
-     * Constructor, initializes all members with the provided objects,
+     * @brief Constructor, initializes all members with the provided objects,
      * initializes temporary storage objects and the klu solver
      *
      * @param t pointer to time variable
      * @param x pointer to state variables
-     * @param model pointer to the AMICI model object
-     * @param rdata pointer to the return data object
+     * @param model pointer to the model object
      */
-    NewtonSolverSparse(realtype *t, AmiVector *x, Model *model, ReturnData *rdata);
+    NewtonSolverSparse(realtype *t, AmiVector *x, Model *model);
     ~NewtonSolverSparse() override;
 
     /**
-     * Solves the linear system for the Newton step
+     * @brief Solves the linear system for the Newton step
      *
      * @param rhs containing the RHS of the linear system, will be
      * overwritten by solution to the linear system
@@ -204,7 +194,7 @@ class NewtonSolverSparse : public NewtonSolver {
     void solveLinearSystem(AmiVector &rhs) override;
 
     /**
-     * Writes the Jacobian for the Newton iteration and passes it to the linear
+     * @brief Writes the Jacobian for the Newton iteration and passes it to the linear
      * solver
      *
      * @param ntry integer newton_try integer start number of Newton solver
@@ -230,17 +220,16 @@ class NewtonSolverIterative : public NewtonSolver {
 
   public:
     /**
-     * Constructor, initializes all members with the provided objects
+     * @brief Constructor, initializes all members with the provided objects
      * @param t pointer to time variable
      * @param x pointer to state variables
-     * @param model pointer to the AMICI model object
-     * @param rdata pointer to the return data object
+     * @param model pointer to the model object
      */
-    NewtonSolverIterative(realtype *t, AmiVector *x, Model *model, ReturnData *rdata);
+    NewtonSolverIterative(realtype *t, AmiVector *x, Model *model);
     ~NewtonSolverIterative() override = default;
 
     /**
-     * Solves the linear system for the Newton step by passing it to
+     * @brief Solves the linear system for the Newton step by passing it to
      * linsolveSPBCG
      *
      * @param rhs containing the RHS of the linear system, will be
