@@ -119,23 +119,12 @@ class Solver {
                 const AmiVector &dxB0, const AmiVector &xQB0) const;
 
     /**
-     * @brief Extracts diagnosis information from solver memory block and
-     * writes them into the return data object
+     * @brief Reinitializes state and respective sensitivities (if necessary) according
+     * to changes in fixedParameters
      *
-     * @param it time-point index
-     * @param rdata pointer to the return data object
+     * @param model pointer to the model instance
      */
-    void getDiagnosis(int it, ReturnData *rdata) const;
-
-    /**
-     * @brief Extracts diagnosis information from solver memory block and
-     * writes them into the return data object for the backward problem
-     *
-     * @param it time-point index
-     * @param rdata pointer to the return data object
-     * @param which identifier of the backwards problem
-     */
-    void getDiagnosisB(int it, ReturnData *rdata, int which) const;
+    void updateAndReinitStatesAndSensitivities(Model *model);
 
     /**
      * getRootInfo extracts information which event occured
@@ -188,6 +177,11 @@ class Solver {
      */
     void setSensitivityMethod(SensitivityMethod sensi_meth);
 
+    /**
+     * @brief Disable forward sensitivity integration (used in steady state sim)
+     */
+    void switchForwardSensisOff() const;
+
     /**
      * @brief Get maximum number of allowed Newton steps for steady state
      * computation
@@ -550,6 +544,18 @@ class Solver {
      * @param ism internal sensitivity method
      */
     void setInternalSensitivityMethod(InternalSensitivityMethod ism);
+    
+    /**
+     * @brief returns the ReturnData reporting mode
+     * @return ReturnData reporting mode
+     */
+    RDataReporting getReturnDataReportingMode() const;
+
+    /**
+     * @brief sets the ReturnData reporting mode
+     * @param rdrm ReturnData reporting mode
+     */
+    void setReturnDataReportingMode(RDataReporting rdrm);
 
     /**
      * @brief write solution from forward simulation
@@ -637,6 +643,12 @@ class Solver {
     virtual void sensReInit(const AmiVectorArray &yyS0,
                             const AmiVectorArray &ypS0) const = 0;
 
+    /**
+     * @brief Switches off computation of  state sensitivites without
+     * deallocating the memory for sensitivities
+     */
+    virtual void sensToggleOff() const = 0;
+
     /**
      * @brief Reinitializes the adjoint states after an event occurence
      *
@@ -691,6 +703,113 @@ class Solver {
      * @return xQB.getLength()
      */
     int nquad() const;
+    
+    /**
+     * @brief check if FSA is being computed
+     * @return flag
+     */
+    bool computingFSA() const {
+        return getSensitivityOrder() >= SensitivityOrder::first &&
+        getSensitivityMethod() == SensitivityMethod::forward;
+    }
+    
+    /**
+     * @brief check if ASA is being computed
+     * @return flag
+     */
+    bool computingASA() const {
+        return getSensitivityOrder() >= SensitivityOrder::first &&
+        getSensitivityMethod() == SensitivityMethod::adjoint;
+    }
+    
+    /**
+     * @brief Resets vectors containing diagnosis information
+     */
+    void resetDiagnosis() const;
+    
+    /**
+     * @brief Stores diagnosis information from solver memory block for forward problem
+     */
+    void storeDiagnosis() const;
+
+    /**
+     * @brief Stores diagnosis information from solver memory block for backward problem
+     *
+     * @param which identifier of the backwards problem
+     */
+    void storeDiagnosisB(int which) const;
+    
+    /**
+     * @brief Accessor ns
+     * @return ns
+     */
+    std::vector<int> const& getNumSteps() const {
+        return ns;
+    }
+    
+    /**
+     * @brief Accessor nsB
+     * @return nsB
+     */
+    std::vector<int> const& getNumStepsB() const {
+        return nsB;
+    }
+    
+    /**
+     * @brief Accessor nrhs
+     * @return nrhs
+     */
+    std::vector<int> const& getNumRhsEvals() const {
+        return nrhs;
+    }
+    
+    /**
+     * @brief Accessor nrhsB
+     * @return nrhsB
+     */
+    std::vector<int> const& getNumRhsEvalsB() const {
+        return nrhsB;
+    }
+    
+    /**
+     * @brief Accessor netf
+     * @return netf
+     */
+    std::vector<int> const& getNumErrTestFails() const {
+        return netf;
+    }
+    
+    /**
+     * @brief Accessor netfB
+     * @return netfB
+     */
+    std::vector<int> const& getNumErrTestFailsB() const {
+        return netfB;
+    }
+    
+    /**
+     * @brief Accessor nnlscf
+     * @return nnlscf
+     */
+    std::vector<int> const& getNumNonlinSolvConvFails() const {
+        return nnlscf;
+    }
+    
+    /**
+     * @brief Accessor nnlscfB
+     * @return nnlscfB
+     */
+    std::vector<int> const& getNumNonlinSolvConvFailsB() const {
+        return nnlscfB;
+    }
+    
+    /**
+     * @brief Accessor order
+     * @return order
+     */
+    std::vector<int> const& getLastOrder() const {
+        return order;
+    }
 
     /**
      * @brief Serialize Solver (see boost::serialization::serialize)
@@ -1344,6 +1463,11 @@ class Solver {
      */
     void setSensInitDone() const;
 
+    /**
+     * @brief sets that memory for forward sensitivities has not been allocated
+     */
+    void setSensInitOff() const;
+
     /**
      * @brief sets that memory for forward interpolation has been allocated
      */
@@ -1460,6 +1584,8 @@ class Solver {
 
     /** relative tolerances for steadystate computation */
     realtype ss_rtol_sensi = NAN;
+    
+    RDataReporting rdata_mode = RDataReporting::full;
 
     /** CPU time, forward solve */
     mutable realtype cpu_time = 0.0;
@@ -1492,6 +1618,37 @@ class Solver {
 
     /** number of checkpoints in the forward problem */
     mutable int ncheckPtr = 0;
+    
+    /** number of integration steps forward problem (dimension: nt) */
+    mutable std::vector<int> ns;
+
+    /** number of integration steps backward problem (dimension: nt) */
+    mutable std::vector<int> nsB;
+
+    /** number of right hand side evaluations forward problem (dimension: nt) */
+    mutable std::vector<int> nrhs;
+
+    /** number of right hand side evaluations backward problem (dimension: nt) */
+    mutable std::vector<int> nrhsB;
+
+    /** number of error test failures forward problem (dimension: nt) */
+    mutable std::vector<int> netf;
+
+    /** number of error test failures backward problem (dimension: nt) */
+    mutable std::vector<int> netfB;
+
+    /**
+     * number of linear solver convergence failures forward problem (dimension:
+     * nt) */
+    mutable std::vector<int> nnlscf;
+
+    /**
+     * number of linear solver convergence failures backward problem (dimension:
+     * nt) */
+    mutable std::vector<int> nnlscfB;
+
+    /** employed order forward problem (dimension: nt) */
+    mutable std::vector<int> order;
 };
 
 bool operator==(const Solver &a, const Solver &b);
