@@ -10,6 +10,7 @@ import shlex
 import subprocess
 import shutil
 
+from distutils import log
 from .swig import find_swig, get_swig_version
 
 try:
@@ -73,12 +74,17 @@ def get_blas_config() -> PackageInfo:
 
     # Try pkgconfig
     if pkgconfig:
-        if pkgconfig.exists('cblas'):
-            blaspkgcfg = pkgconfig.parse('cblas')
-            blaspkgcfg['extra_compile_args'] = [pkgconfig.cflags('cblas')]
-            blaspkgcfg['extra_link_args'] = [pkgconfig.libs('cblas')]
+        for blas_name in ['cblas', 'openblas']:
+            if pkgconfig.exists(blas_name):
+                blaspkgcfg = pkgconfig.parse(blas_name)
+                blaspkgcfg['extra_compile_args'] = [
+                    pkgconfig.cflags(blas_name)
+                ]
+                blaspkgcfg['extra_link_args'] = [
+                    pkgconfig.libs(blas_name)
+                ]
 
-            return blaspkgcfg
+                return blaspkgcfg
 
     # If none of the previous worked, fall back to libcblas in default paths
     blaspkgcfg['libraries'] = ['cblas']
@@ -93,18 +99,6 @@ def get_hdf5_config() -> PackageInfo:
     :return:
         hdf5 related package information
     """
-    if pkgconfig:
-        h5pkgcfg = {}
-        try:
-            h5pkgcfg = pkgconfig.parse('hdf5')
-        except pkgconfig.PackageNotFoundError:
-            pass
-        # NOTE: Cannot use pkgconfig.exists('hdf5f'), since this is true
-        # although no libraries or include dirs are available
-        h5pkgcfg['found'] = 'include_dirs' in h5pkgcfg \
-                            and h5pkgcfg['include_dirs']
-        if h5pkgcfg['found']:
-            return h5pkgcfg
 
     h5pkgcfg = {'include_dirs': [],
                 'library_dirs': [],
@@ -129,13 +123,6 @@ def get_hdf5_config() -> PackageInfo:
         '/usr/local/Cellar/hdf5/1.10.2_1/lib'  # travis macOS
     ]
 
-    # Check for Environment Modules variables
-    if 'HDF5_BASE' in os.environ:
-        hdf5_include_dir_hints.insert(
-            0, os.path.join(os.environ['HDF5_BASE'], 'include'))
-        hdf5_library_dir_hints.insert(
-            0, os.path.join(os.environ['HDF5_BASE'], 'lib'))
-
     # special treatment for conda environments
     # as the conda library dir is provided first, we should also check for
     # conda header files first
@@ -145,11 +132,18 @@ def get_hdf5_config() -> PackageInfo:
         hdf5_library_dir_hints.insert(
             0, os.path.join(os.environ['CONDA_DIR'], 'lib'))
 
+    # Check for Environment Modules variables
+    if 'HDF5_BASE' in os.environ:
+        hdf5_include_dir_hints.insert(
+            0, os.path.join(os.environ['HDF5_BASE'], 'include'))
+        hdf5_library_dir_hints.insert(
+            0, os.path.join(os.environ['HDF5_BASE'], 'lib'))
+
     for hdf5_include_dir_hint in hdf5_include_dir_hints:
         hdf5_include_dir_found = os.path.isfile(
             os.path.join(hdf5_include_dir_hint, 'hdf5.h'))
         if hdf5_include_dir_found:
-            print('hdf5.h found in %s' % hdf5_include_dir_hint)
+            log.info('hdf5.h found in %s' % hdf5_include_dir_hint)
             h5pkgcfg['include_dirs'] = [hdf5_include_dir_hint]
             break
 
@@ -159,13 +153,28 @@ def get_hdf5_config() -> PackageInfo:
             hdf5_library_dir_found = os.path.isfile(
                 os.path.join(hdf5_library_dir_hint, lib_filename))
             if hdf5_library_dir_found:
-                print(f'{lib_filename} found in {hdf5_library_dir_hint}')
+                log.info(f'{lib_filename} found in {hdf5_library_dir_hint}')
                 h5pkgcfg['library_dirs'] = [hdf5_library_dir_hint]
                 break
         if hdf5_library_dir_found:
             # break to not override hdf5_library_dir_found
             break
+
     h5pkgcfg['found'] = hdf5_include_dir_found and hdf5_library_dir_found
+    if h5pkgcfg['found']:
+        return h5pkgcfg
+
+    if pkgconfig:
+        try:
+            h5pkgcfg = pkgconfig.parse('hdf5')
+        except pkgconfig.PackageNotFoundError:
+            pass
+        # NOTE: Cannot use pkgconfig.exists('hdf5f'), since this is true
+        # although no libraries or include dirs are available
+        h5pkgcfg['found'] = 'include_dirs' in h5pkgcfg \
+                            and h5pkgcfg['include_dirs'] and \
+                            'library_dirs' in h5pkgcfg \
+                            and h5pkgcfg['library_dirs']
 
     return h5pkgcfg
 
@@ -183,10 +192,10 @@ def add_coverage_flags_if_required(cxx_flags: List[str],
     """
     if 'ENABLE_GCOV_COVERAGE' in os.environ and \
             os.environ['ENABLE_GCOV_COVERAGE'] == 'TRUE':
-        print("ENABLE_GCOV_COVERAGE was set to TRUE."
+        log.info("ENABLE_GCOV_COVERAGE was set to TRUE."
               " Building AMICI with coverage symbols.")
         cxx_flags.extend(['-g', '-O0',  '--coverage'])
-        linker_flags.extend(['--coverage','-g'])
+        linker_flags.extend(['--coverage', '-g'])
 
 
 def add_debug_flags_if_required(cxx_flags: List[str],
@@ -203,7 +212,7 @@ def add_debug_flags_if_required(cxx_flags: List[str],
     """
     if 'ENABLE_AMICI_DEBUGGING' in os.environ \
             and os.environ['ENABLE_AMICI_DEBUGGING'] == 'TRUE':
-        print("ENABLE_AMICI_DEBUGGING was set to TRUE."
+        log.info("ENABLE_AMICI_DEBUGGING was set to TRUE."
               " Building AMICI with debug symbols.")
         cxx_flags.extend(['-g', '-O0'])
         linker_flags.extend(['-g'])
@@ -225,7 +234,7 @@ def generate_swig_interface_files() -> None:
         '-Iamici/swig', '-Iamici/include',
     ]
 
-    print(f"Found SWIG version {swig_version}")
+    log.info(f"Found SWIG version {swig_version}")
 
     # Swig AMICI interface without HDF5 dependency
     swig_cmd = [swig_exe,
@@ -239,7 +248,7 @@ def generate_swig_interface_files() -> None:
     if swig_version >= (4, 0, 0):
         swig_cmd.insert(1, '-doxygen')
 
-    print(f"Running SWIG: {' '.join(swig_cmd)}")
+    log.info(f"Running SWIG: {' '.join(swig_cmd)}")
     sp = subprocess.run(swig_cmd, stdout=subprocess.PIPE,
                         stderr=sys.stdout.buffer)
     if not sp.returncode == 0:
@@ -256,7 +265,7 @@ def generate_swig_interface_files() -> None:
                 'amici/swig/amici.i']
     if swig_version >= (4, 0, 0):
         swig_cmd.insert(1, '-doxygen')
-    print(f"Running SWIG: {' '.join(swig_cmd)}")
+    log.info(f"Running SWIG: {' '.join(swig_cmd)}")
     sp = subprocess.run(swig_cmd, stdout=subprocess.PIPE,
                         stderr=sys.stdout.buffer)
     if not sp.returncode == 0:
