@@ -5,6 +5,7 @@
 
 #include "amici/amici.h"
 
+#include "amici/steadystateproblem.h"
 #include "amici/backwardproblem.h"
 #include "amici/forwardproblem.h"
 #include "amici/misc.h"
@@ -21,15 +22,18 @@
 #include <type_traits>
 
 // ensure definitions are in sync
-static_assert(AMICI_SUCCESS == CV_SUCCESS, "AMICI_SUCCESS != CV_SUCCESS");
-static_assert(AMICI_DATA_RETURN == CV_TSTOP_RETURN,
+static_assert(amici::AMICI_SUCCESS == CV_SUCCESS,
+              "AMICI_SUCCESS != CV_SUCCESS");
+static_assert(amici::AMICI_DATA_RETURN == CV_TSTOP_RETURN,
               "AMICI_DATA_RETURN != CV_TSTOP_RETURN");
-static_assert(AMICI_ROOT_RETURN == CV_ROOT_RETURN,
+static_assert(amici::AMICI_ROOT_RETURN == CV_ROOT_RETURN,
               "AMICI_ROOT_RETURN != CV_ROOT_RETURN");
-static_assert(AMICI_ILL_INPUT == CV_ILL_INPUT,
+static_assert(amici::AMICI_ILL_INPUT == CV_ILL_INPUT,
               "AMICI_ILL_INPUT != CV_ILL_INPUT");
-static_assert(AMICI_NORMAL == CV_NORMAL, "AMICI_NORMAL != CV_NORMAL");
-static_assert(AMICI_ONE_STEP == CV_ONE_STEP, "AMICI_ONE_STEP != CV_ONE_STEP");
+static_assert(amici::AMICI_NORMAL == CV_NORMAL,
+              "AMICI_NORMAL != CV_NORMAL");
+static_assert(amici::AMICI_ONE_STEP == CV_ONE_STEP,
+              "AMICI_ONE_STEP != CV_ONE_STEP");
 static_assert(std::is_same<amici::realtype, realtype>::value,
               "Definition of realtype does not match");
 
@@ -96,30 +100,58 @@ AmiciApplication::runAmiciSimulation(Solver& solver,
                                      Model& model,
                                      bool rethrow)
 {
-    std::unique_ptr<ReturnData> rdata;
-
     /* Applies condition-specific model settings and restores them when going
      * out of scope */
-    ConditionContext conditionContext(&model, edata);
+    ConditionContext cc1(&model, edata, FixedParameterContext::simulation);
+    
+    std::unique_ptr<ReturnData> rdata = std::make_unique<ReturnData>(solver,
+                                                                     model);
 
+    if (model.nx_solver <= 0) {
+        return rdata;
+    }
+
+    std::unique_ptr<SteadystateProblem> preeq {};
+    std::unique_ptr<ForwardProblem> fwd {};
+    std::unique_ptr<BackwardProblem> bwd {};
+    std::unique_ptr<SteadystateProblem> posteq {};
+    
     try {
-        rdata = std::unique_ptr<ReturnData>(new ReturnData(solver, model));
-
-        if (model.nx_solver <= 0) {
-            return rdata;
+        if (solver.getPreequilibration() ||
+            (edata && !edata->fixedParametersPreequilibration.empty())) {
+            ConditionContext cc2(
+                &model, edata, FixedParameterContext::preequilibration
+            );
+            
+            preeq = std::make_unique<SteadystateProblem>(solver, model);
+            preeq->workSteadyStateProblem(&solver, &model, -1);
         }
 
-        auto fwd = std::unique_ptr<ForwardProblem>(
-          new ForwardProblem(rdata.get(), edata, &model, &solver));
+        
+        fwd = std::make_unique<ForwardProblem>(edata, &model, &solver,
+                                               preeq.get());
         fwd->workForwardProblem();
+        
+        
+        if (fwd->getCurrentTimeIteration() < model.nt()) {
+            posteq = std::make_unique<SteadystateProblem>(solver, model);
+            posteq->workSteadyStateProblem(&solver, &model,
+                                           fwd->getCurrentTimeIteration());
+        }
 
-        auto bwd =
-          std::unique_ptr<BackwardProblem>(new BackwardProblem(fwd.get()));
-        bwd->workBackwardProblem();
-
+        
+        if (edata && solver.computingASA()) {
+            fwd->getAdjointUpdates(model, *edata);
+            if (posteq)
+                posteq->getAdjointUpdates(model, *edata);
+            
+            bwd = std::make_unique<BackwardProblem>(*fwd, posteq.get());
+            bwd->workBackwardProblem();
+        }
+        
         rdata->status = AMICI_SUCCESS;
+        
     } catch (amici::IntegrationFailure const& ex) {
-        rdata->invalidate(ex.time);
         rdata->status = ex.error_code;
         if (rethrow)
             throw;
@@ -128,7 +160,6 @@ AmiciApplication::runAmiciSimulation(Solver& solver,
                  ex.time,
                  ex.what());
     } catch (amici::IntegrationFailureB const& ex) {
-        rdata->invalidateSLLH();
         rdata->status = ex.error_code;
         if (rethrow)
             throw;
@@ -139,7 +170,6 @@ AmiciApplication::runAmiciSimulation(Solver& solver,
           ex.time,
           ex.what());
     } catch (amici::AmiException const& ex) {
-        rdata->invalidate(model.t0());
         rdata->status = AMICI_ERROR;
         if (rethrow)
             throw;
@@ -149,7 +179,8 @@ AmiciApplication::runAmiciSimulation(Solver& solver,
                  ex.getBacktrace());
     }
 
-    rdata->applyChainRuleFactorToSimulationResults(&model);
+    rdata->processSimulationObjects(preeq.get(), fwd.get(), bwd.get(),
+                                    posteq.get(), model, solver, edata);
 
     return rdata;
 }
