@@ -368,19 +368,46 @@ void writeReturnDataDiagnosis(const ReturnData &rdata,
                     rdata.numnonlinsolvconvfailsB);
 
     H5LTset_attribute_int(file.getId(), (hdf5Location + "").c_str(),
-                          "newton_status", &rdata.newton_status, 1);
+                          "preeq_status", &rdata.preeq_status, 1);
 
-    if (!rdata.newton_numsteps.empty())
-        createAndWriteInt1DDataset(file, hdf5Location + "/newton_numsteps",
-                                   rdata.newton_numsteps);
+    if (!rdata.preeq_numsteps.empty())
+        createAndWriteInt1DDataset(file, hdf5Location + "/preeq_numsteps",
+                                   rdata.preeq_numsteps);
 
-    if (!rdata.newton_numlinsteps.empty())
-        createAndWriteInt2DDataset(file, hdf5Location + "/newton_numlinsteps",
-                                   rdata.newton_numlinsteps,
+    if (!rdata.preeq_numlinsteps.empty())
+        createAndWriteInt2DDataset(file, hdf5Location + "/preeq_numlinsteps",
+                                   rdata.preeq_numlinsteps,
                                    rdata.newton_maxsteps, 2);
 
     H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
-                             "newton_cpu_time", &rdata.newton_cpu_time, 1);
+                             "preeq_cpu_time", &rdata.preeq_cpu_time, 1);
+
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(), "preeq_t",
+                             &rdata.preeq_t, 1);
+
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(), "preeq_wrms",
+                             &rdata.preeq_wrms, 1);
+
+    H5LTset_attribute_int(file.getId(), (hdf5Location + "").c_str(),
+                          "posteq_status", &rdata.posteq_status, 1);
+
+    if (!rdata.posteq_numsteps.empty())
+        createAndWriteInt1DDataset(file, hdf5Location + "/posteq_numsteps",
+                                   rdata.posteq_numsteps);
+
+    if (!rdata.posteq_numlinsteps.empty())
+        createAndWriteInt2DDataset(file, hdf5Location + "/posteq_numlinsteps",
+                                   rdata.posteq_numlinsteps,
+                                   rdata.newton_maxsteps, 2);
+
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "posteq_cpu_time", &rdata.posteq_cpu_time, 1);
+
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(), "posteq_t",
+                             &rdata.posteq_t, 1);
+
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(), "posteq_wrms",
+                             &rdata.posteq_wrms, 1);
 
     H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
                              "cpu_time", &rdata.cpu_time, 1);
@@ -513,6 +540,140 @@ bool attributeExists(H5::H5Object const& object,
     return result > 0;
 }
 
+void writeSolverSettingsToHDF5(Solver const& solver,
+                              std::string const& hdf5Filename,
+                              std::string const& hdf5Location) {
+    auto file = createOrOpenForWriting(hdf5Filename);
+
+    writeSolverSettingsToHDF5(solver, file, hdf5Location);
+}
+
+void writeSolverSettingsToHDF5(Solver const& solver,
+                              H5::H5File const& file,
+                              const std::string& hdf5Location) {
+    if(!locationExists(file, hdf5Location))
+        createGroup(file, hdf5Location);
+
+    double dbuffer;
+    int ibuffer;
+
+    dbuffer = solver.getAbsoluteTolerance();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "atol", &dbuffer, 1);
+
+    dbuffer = solver.getRelativeTolerance();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "rtol", &dbuffer, 1);
+
+    dbuffer = solver.getAbsoluteToleranceFSA();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "atol_fsa", &dbuffer, 1);
+
+    dbuffer = solver.getRelativeToleranceFSA();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "rtol_fsa", &dbuffer, 1);
+
+    dbuffer = solver.getAbsoluteToleranceB();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "atolB", &dbuffer, 1);
+
+    dbuffer = solver.getRelativeToleranceB();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "rtolB", &dbuffer, 1);
+
+    dbuffer = solver.getAbsoluteToleranceQuadratures();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "quad_atol", &dbuffer, 1);
+
+    dbuffer = solver.getRelativeToleranceQuadratures();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "quad_rtol", &dbuffer, 1);
+
+    dbuffer = solver.getAbsoluteToleranceSteadyState();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "ss_atol", &dbuffer, 1);
+
+    dbuffer = solver.getRelativeToleranceSteadyState();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "ss_rtol", &dbuffer, 1);
+
+    dbuffer = solver.getAbsoluteToleranceSteadyStateSensi();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "ss_atol_sensi", &dbuffer, 1);
+
+    dbuffer = solver.getRelativeToleranceSteadyStateSensi();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "ss_rtol_sensi", &dbuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getMaxSteps());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "maxsteps", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getMaxStepsBackwardProblem());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "maxstepsB", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getLinearMultistepMethod());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "lmm", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getNonlinearSolverIteration());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "iter", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getStabilityLimitFlag());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "stldet", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getStateOrdering());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "ordering", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getInterpolationType());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "interpType", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getSensitivityMethod());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "sensi_meth", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getSensitivityOrder());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "sensi", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getNewtonMaxSteps());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "newton_maxsteps", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getPreequilibration());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "newton_preeq", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getNewtonDampingFactorMode());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "newton_damping_factor_mode", &ibuffer, 1);
+
+    dbuffer = solver.getNewtonDampingFactorLowerBound();
+    H5LTset_attribute_double(file.getId(), hdf5Location.c_str(),
+                             "newton_damping_factor_lower_bound", &dbuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getNewtonMaxLinearSteps());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "newton_maxlinsteps", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getLinearSolver());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "linsol", &ibuffer, 1);
+
+    ibuffer = static_cast<int>(solver.getInternalSensitivityMethod());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "ism", &ibuffer, 1);
+    
+    ibuffer = static_cast<int>(solver.getReturnDataReportingMode());
+    H5LTset_attribute_int(file.getId(), hdf5Location.c_str(),
+                          "rdrm", &ibuffer, 1);
+}
+
 void readSolverSettingsFromHDF5(H5::H5File const& file, Solver &solver,
                                 const std::string &datasetPath) {
 
@@ -668,6 +829,12 @@ void readSolverSettingsFromHDF5(H5::H5File const& file, Solver &solver,
                     static_cast<InternalSensitivityMethod>(
                         getIntScalarAttribute(file, datasetPath, "ism")));
     }
+    
+    if(attributeExists(file, datasetPath, "rdrm")) {
+        solver.setReturnDataReportingMode(
+                    static_cast<RDataReporting>(
+                        getIntScalarAttribute(file, datasetPath, "rdrm")));
+    }
 }
 
 void readSolverSettingsFromHDF5(const std::string &hdffile, Solver &solver,
