@@ -9,14 +9,21 @@
 #include "sundials/sundials_types.h"
 
 namespace amici {
-class Solver;
-}
 
-/**
- * @brief Wrapper function to instantiate the linked Amici model without knowing
- * the name at compile time.
- * @return
- */
+class Solver;
+
+namespace model_TPL_MODELNAME {
+
+extern std::array<std::string, TPL_NP> parameterNames;
+extern std::array<std::string, TPL_NK> fixedParameterNames;
+extern std::array<std::string, TPL_NX_RDATA> stateNames;
+extern std::array<std::string, TPL_NY> observableNames;
+extern std::array<std::string, TPL_NP> parameterIds;
+extern std::array<std::string, TPL_NK> fixedParameterIds;
+extern std::array<std::string, TPL_NX_RDATA> stateIds;
+extern std::array<std::string, TPL_NY> observableIds;
+
+
 extern void J_TPL_MODELNAME(realtype *J, const realtype t, const realtype *x,
                             const realtype *p, const realtype *k,
                             const realtype *h, const realtype *w,
@@ -151,7 +158,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param w vector with helper variables
      * @param dwdx derivative of w wrt x
      **/
@@ -167,7 +174,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param xB Vector with the adjoint states
      * @param w vector with helper variables
      * @param dwdx derivative of w wrt x
@@ -185,7 +192,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param w vector with helper variables
      * @param dwdx derivative of w wrt x
      **/
@@ -326,7 +333,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param ip sensitivity index
      * @param ie event index
      * @param xdot new model right hand side
@@ -346,7 +353,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param w repeating elements vector
      * @param ip sensitivity index
      * @param ie event index
@@ -368,7 +375,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param ie event index
      * @param xdot new model right hand side
      * @param xdot_old previous model right hand side
@@ -384,7 +391,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param ie event index
      * @param xdot new model right hand side
      * @param xdot_old previous model right hand side
@@ -404,7 +411,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param ip parameter index w.r.t. which the derivative is requested
      **/
     virtual void fdrzdp(realtype *drzdp, const int ie, const realtype t,
@@ -418,7 +425,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      **/
     virtual void fdrzdx(realtype *drzdx, const int ie, const realtype t,
                         const realtype *x, const realtype *p, const realtype *k,
@@ -454,38 +461,38 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
     TPL_DJYDY_ROWVALS_IMPL
 
     TPL_DWDP_IMPL
-    
+
     TPL_DWDP_COLPTRS_IMPL
-    
+
     TPL_DWDP_ROWVALS_IMPL
 
     TPL_DWDX_IMPL
 
     TPL_DWDX_COLPTRS_IMPL
-    
+
     TPL_DWDX_ROWVALS_IMPL
-    
+
     TPL_DXDOTDW_IMPL
     TPL_DXDOTDW_COLPTRS_IMPL
     TPL_DXDOTDW_ROWVALS_IMPL
 
     TPL_DXDOTDP_EXPLICIT_IMPL
-    
+
     TPL_DXDOTDP_EXPLICIT_COLPTRS_IMPL
-    
+
     TPL_DXDOTDP_EXPLICIT_ROWVALS_IMPL
 
     TPL_DXDOTDP_IMPLICIT_COLPTRS_IMPL
-    
+
     TPL_DXDOTDP_IMPLICIT_ROWVALS_IMPL
-    
+
     /** model specific implementation of fdydx
      * @param dydx partial derivative of observables y w.r.t. model states x
      * @param t current time
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      **/
     virtual void fdydx(realtype *dydx, const realtype t, const realtype *x,
                        const realtype *p, const realtype *k, const realtype *h,
@@ -499,7 +506,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param ip parameter index w.r.t. which the derivative is requested
      **/
     virtual void fdydp(realtype *dydp, const realtype t, const realtype *x,
@@ -517,7 +524,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param ip parameter index w.r.t. which the derivative is requested
      **/
     virtual void fdzdp(realtype *dzdp, const int ie, const realtype t,
@@ -532,7 +539,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      **/
     virtual void fdzdx(realtype *dzdx, const int ie, const realtype t,
                        const realtype *x, const realtype *p, const realtype *k,
@@ -544,7 +551,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      **/
     virtual void froot(realtype *root, const realtype t, const realtype *x,
                        const realtype *p, const realtype *k,
@@ -558,7 +565,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      **/
     virtual void frz(realtype *rz, const int ie, const realtype t,
                      const realtype *x, const realtype *p, const realtype *k,
@@ -592,7 +599,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param p parameter vector
      * @param k constant vector
      * @param sx current state sensitivity
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param ip sensitivity index
      **/
     virtual void fsrz(realtype *srz, const int ie, const realtype t,
@@ -606,7 +613,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param sx current state sensitivity
      * @param ip sensitivity index
      * @param ie event index
@@ -652,7 +659,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param sx current state sensitivity
      * @param ip sensitivity index
      **/
@@ -692,7 +699,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      * @param w vector with helper variables
      **/
     virtual void fxdot(realtype *xdot, const realtype t, const realtype *x,
@@ -707,7 +714,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      **/
     virtual void fy(realtype *y, const realtype t, const realtype *x,
                     const realtype *p, const realtype *k, const realtype *h,
@@ -722,7 +729,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @param x current state
      * @param p parameter vector
      * @param k constant vector
-     * @param h heavyside vector
+     * @param h heaviside vector
      **/
     virtual void fz(realtype *z, const int ie, const realtype t,
                     const realtype *x, const realtype *p, const realtype *k,
@@ -733,7 +740,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
     TPL_X_SOLVER_IMPL
 
     TPL_TOTAL_CL_IMPL
-    
+
     std::string getName() const override {
         return "TPL_MODELNAME";
     }
@@ -743,7 +750,8 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @return the names
      */
     virtual std::vector<std::string> getParameterNames() const override {
-        return std::vector<std::string>{TPL_PARAMETER_NAMES_INITIALIZER_LIST};
+        return std::vector<std::string>(parameterNames.begin(),
+                                        parameterNames.end());
     }
 
     /**
@@ -751,7 +759,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @return the names
      */
     virtual std::vector<std::string> getStateNames() const override {
-        return std::vector<std::string>{TPL_STATE_NAMES_INITIALIZER_LIST};
+        return std::vector<std::string>(stateNames.begin(), stateNames.end());
     }
 
     /**
@@ -759,8 +767,8 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @return the names
      */
     virtual std::vector<std::string> getFixedParameterNames() const override {
-        return std::vector<std::string>{
-            TPL_FIXED_PARAMETER_NAMES_INITIALIZER_LIST};
+        return std::vector<std::string>(fixedParameterNames.begin(),
+                                        fixedParameterNames.end());
     }
 
     /**
@@ -768,7 +776,8 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @return the names
      */
     virtual std::vector<std::string> getObservableNames() const override {
-        return std::vector<std::string>{TPL_OBSERVABLE_NAMES_INITIALIZER_LIST};
+        return std::vector<std::string>(observableNames.begin(),
+                                        observableNames.end());
     }
 
     /**
@@ -776,7 +785,8 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @return the ids
      */
     virtual std::vector<std::string> getParameterIds() const override {
-        return std::vector<std::string>{TPL_PARAMETER_IDS_INITIALIZER_LIST};
+        return std::vector<std::string>(parameterIds.begin(),
+                                        parameterIds.end());
     }
 
     /**
@@ -784,7 +794,7 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @return the ids
      */
     virtual std::vector<std::string> getStateIds() const override {
-        return std::vector<std::string>{TPL_STATE_IDS_INITIALIZER_LIST};
+        return std::vector<std::string>(stateIds.begin(), stateIds.end());
     }
 
     /**
@@ -792,8 +802,8 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @return the ids
      */
     virtual std::vector<std::string> getFixedParameterIds() const override {
-        return std::vector<std::string>{
-            TPL_FIXED_PARAMETER_IDS_INITIALIZER_LIST};
+        return std::vector<std::string>(fixedParameterIds.begin(),
+                                        fixedParameterIds.end());
     }
 
     /**
@@ -801,7 +811,8 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
      * @return the ids
      */
     virtual std::vector<std::string> getObservableIds() const override {
-        return std::vector<std::string>{TPL_OBSERVABLE_IDS_INITIALIZER_LIST};
+        return std::vector<std::string>(observableIds.begin(),
+                                        observableIds.end());
     }
 
     /**
@@ -831,4 +842,9 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
     }
 };
 
+
+} // namespace model_TPL_MODELNAME
+
+} // namespace amici
+
 #endif /* _amici_TPL_MODELNAME_h */
