@@ -89,8 +89,9 @@ static int fsxdot(int Ns, realtype t, N_Vector x, N_Vector xdot, int ip,
 void CVodeSolver::init(const realtype t0, const AmiVector &x0,
                        const AmiVector & /*dx0*/) const {
     solverWasCalledF = false;
+    forceReInitPostProcessF = false;
     t = t0;
-    x.copy(x0);
+    x = x0;
     int status;
     if (getInitDone()) {
         status = CVodeReInit(solverMemory.get(), t0, x.getNVector());
@@ -104,17 +105,21 @@ void CVodeSolver::init(const realtype t0, const AmiVector &x0,
 
 void CVodeSolver::sensInit1(const AmiVectorArray &sx0,
                             const AmiVectorArray & /*sdx0*/) const {
-    int status;
-    sx.copy(sx0);
-    if (getSensInitDone()) {
-        status = CVodeSensReInit(solverMemory.get(),
-                                 static_cast<int>(getSensitivityMethod()),
-                                 sx.getNVectorArray());
-    } else {
-        status = CVodeSensInit1(solverMemory.get(), nplist(),
-                                static_cast<int>(getSensitivityMethod()),
-                                fsxdot, sx.getNVectorArray());
-        setSensInitDone();
+    int status = CV_SUCCESS;
+    sx = sx0;
+    if (getSensitivityMethod() == SensitivityMethod::forward && nplist() > 0) {
+        if (getSensInitDone()) {
+            status = CVodeSensReInit(
+                solverMemory.get(),
+                static_cast<int>(getInternalSensitivityMethod()),
+                sx.getNVectorArray());
+        } else {
+            status =
+                CVodeSensInit1(solverMemory.get(), nplist(),
+                               static_cast<int>(getInternalSensitivityMethod()),
+                               fsxdot, sx.getNVectorArray());
+            setSensInitDone();
+        }
     }
     if (status != CV_SUCCESS)
         throw CvodeException(status, "CVodeSensInit1");
@@ -124,7 +129,8 @@ void CVodeSolver::binit(const int which, const realtype tf,
                         const AmiVector &xB0,
                         const AmiVector & /*dxB0*/) const {
     solverWasCalledB = false;
-    xB.copy(xB0);
+    forceReInitPostProcessB = false;
+    xB = xB0;
     int status;
     if (getInitDoneB(which)) {
         status = CVodeReInitB(solverMemory.get(), which, tf, xB.getNVector());
@@ -138,7 +144,7 @@ void CVodeSolver::binit(const int which, const realtype tf,
 }
 
 void CVodeSolver::qbinit(const int which, const AmiVector &xQB0) const {
-    xQB.copy(xQB0);
+    xQB = xQB0;
     int status;
     if (getQuadInitDoneB(which)) {
         status = CVodeQuadReInitB(solverMemory.get(), which, xQB.getNVector());
@@ -344,7 +350,7 @@ void CVodeSolver::setStabLimDetB(const int which, const int stldet) const {
         throw CvodeException(status, "CVodeSetStabLimDetB");
 }
 
-void CVodeSolver::setId(const Model */*model*/) const {}
+void CVodeSolver::setId(const Model * /*model*/) const {}
 
 void CVodeSolver::setSuppressAlg(const bool /*flag*/) const {}
 
@@ -486,6 +492,12 @@ void CVodeSolver::reInitB(const int which, const realtype tB0,
     resetState(cv_memB, xB.getNVector());
 }
 
+void CVodeSolver::sensToggleOff() const {
+    auto status = CVodeSensToggleOff(solverMemory.get());
+    if (status != CV_SUCCESS)
+        throw CvodeException(status, "CVodeSensToggleOff");
+}
+
 void CVodeSolver::quadReInitB(int which, const AmiVector &yQB0) const {
     auto cv_memB =
         static_cast<CVodeMem>(CVodeGetAdjCVodeBmem(solverMemory.get(), which));
@@ -572,7 +584,7 @@ void CVodeSolver::allocateSolverB(int *which) const {
         solverMemoryB.resize(*which + 1);
     solverMemoryB.at(*which) =
         std::unique_ptr<void, std::function<void(void *)>>(
-            getAdjBmem(solverMemory.get(), *which), [](void */*ptr*/) {});
+            getAdjBmem(solverMemory.get(), *which), [](void * /*ptr*/) {});
     if (status != CV_SUCCESS)
         throw CvodeException(status, "CVodeCreateB");
 }
