@@ -201,8 +201,10 @@ void SUNMatrixWrapper::multiply(gsl::span<realtype> c,
 
     switch (SUNMatGetID(matrix)) {
     case SUNMATRIX_DENSE:
-        amici_dgemv(BLASLayout::colMajor, BLASTranspose::noTrans, nrows,
-                    ncols, 1.0, data(), nrows, b.data(), 1, 1.0, c.data(), 1);
+        amici_dgemv(BLASLayout::colMajor, BLASTranspose::noTrans,
+                    static_cast<int>(nrows), static_cast<int>(ncols),
+                    1.0, data(), static_cast<int>(nrows),
+                    b.data(), 1, 1.0, c.data(), 1);
         break;
     case SUNMATRIX_SPARSE:
 
@@ -230,7 +232,12 @@ void SUNMatrixWrapper::multiply(gsl::span<realtype> c,
     case SUNMATRIX_CUSTOM:
         throw std::domain_error("Amici currently does not support custom"
                                 " matrix types.");
+    case SUNMATRIX_SLUNRLOC:
+        throw std::domain_error("Not Implemented.");
+    case SUNMATRIX_CUSPARSE:
+        throw std::domain_error("Not Implemented.");
     }
+
 }
 
 void SUNMatrixWrapper::multiply(N_Vector c,
@@ -392,6 +399,10 @@ void SUNMatrixWrapper::update_ptrs() {
     case SUNMATRIX_CUSTOM:
         throw std::domain_error("Amici currently does not support "
                                 "custom matrix types.");
+    case SUNMATRIX_SLUNRLOC:
+        throw std::domain_error("Not Implemented.");
+    case SUNMATRIX_CUSPARSE:
+        throw std::domain_error("Not Implemented.");
     }
 }
 
