@@ -43,23 +43,17 @@ def get_sundials_sources() -> List[str]:
         os.path.join('src', 'idas', 'idas_nls_sim.c'),
         os.path.join('src', 'idas', 'idaa_io.c'),
         os.path.join('src', 'sundials', 'sundials_math.c'),
-        os.path.join('src', 'sundials', 'sundials_mpi.c'),
-        os.path.join('src', 'sundials', 'sundials_sptfqmr.c'),
         os.path.join('src', 'sundials', 'sundials_matrix.c'),
-        os.path.join('src', 'sundials', 'sundials_pcg.c'),
         os.path.join('src', 'sundials', 'sundials_direct.c'),
-        os.path.join('src', 'sundials', 'sundials_spgmr.c'),
-        os.path.join('src', 'sundials', 'sundials_spbcgs.c'),
         os.path.join('src', 'sundials', 'sundials_nvector_senswrapper.c'),
         os.path.join('src', 'sundials', 'sundials_dense.c'),
         os.path.join('src', 'sundials', 'sundials_nvector.c'),
         os.path.join('src', 'sundials', 'sundials_version.c'),
-        os.path.join('src', 'sundials', 'sundials_spfgmr.c'),
-        os.path.join('src', 'sundials', 'sundials_sparse.c'),
         os.path.join('src', 'sundials', 'sundials_iterative.c'),
         os.path.join('src', 'sundials', 'sundials_nonlinearsolver.c'),
         os.path.join('src', 'sundials', 'sundials_linearsolver.c'),
         os.path.join('src', 'sundials', 'sundials_band.c'),
+        os.path.join('src', 'sundials', 'sundials_futils.c'),
         os.path.join('src', 'sunnonlinsol', 'newton', 'sunnonlinsol_newton.c'),
         os.path.join('src', 'sunnonlinsol', 'fixedpoint',
                      'sunnonlinsol_fixedpoint.c'),
@@ -135,9 +129,9 @@ def get_amici_base_sources(with_hdf5: bool = True) -> List[str]:
         with_hdf5: compile with HDF5 support
     """
 
-    amici_base_sources = glob.glob('amici{s}src{s}*.cpp'.format(s=os.sep))
-    amici_base_sources = [src for src in amici_base_sources if not re.search(
-        r'(matlab)|(\.template\.)', src)]
+    amici_base_sources = glob.glob(os.path.join('amici', 'src', '*.cpp'))
+    amici_base_sources = [src for src in amici_base_sources
+                          if not re.search(r'(matlab)|(\.(ODE_)?template\.)', src)]
 
     if not with_hdf5:
         hdf5_cpp = os.path.join('amici', 'src', 'hdf5.cpp')
@@ -197,8 +191,7 @@ def get_lib_suite_sparse(extra_compiler_flags: Optional[List[str]] = None) -> \
                          'amici/ThirdParty/SuiteSparse/COLAMD/Include/',
                          'amici/ThirdParty/SuiteSparse/BTF/Include/',
                          'amici/ThirdParty/SuiteSparse/SuiteSparse_config',
-                         'amici/ThirdParty/SuiteSparse/include'
-                         ],
+                         'amici/ThirdParty/SuiteSparse/include'],
         'cflags': [*extra_compiler_flags],
         'cflags_mingw32': ['-Wno-unused-but-set-variable'],
         'cflags_unix': ['-Wno-unused-but-set-variable']
